import 'dart:io';

import 'package:global/global.dart';
import 'package:http/http.dart' as http;

class GHttpReqs {
  //Utility function for making http requests
  static Future<Response> send({
    bool showDialog = true,
    bool isPostRequest = true,
    required String url,
    required Map<String, String>? Function(String url) getHeaders,
    Object? body,
    Function(Response)? onResponse,
    Function? onError,
  }) {
    if (showDialog) {
      GUtils.showProgressDialog();
    }
    Future<Response> responseFuture = isPostRequest
        ? http.post(Uri.parse(url), body: body, headers: getHeaders(url))
        : http.get(Uri.parse(url), headers: getHeaders(url));
    _attachHandlerToBaseResponseFuture(
        baseRespFuture: responseFuture,
        showDialog: showDialog,
        onResponse: onResponse,
        onError: onError);
    return responseFuture;
  }

  static _attachHandlerToBaseResponseFuture({
    required Future<BaseResponse> baseRespFuture,
    bool showDialog = true,
    Function(Response)? onResponse,
    Function? onError,
  }) =>
      baseRespFuture.then(
        (baseResp) async {
          if (showDialog) Global.navigatorKey.currentState!.pop();
          if (onResponse != null) {
            onResponse(baseResp is StreamedResponse
                ? await Response.fromStream(baseResp)
                : baseResp as Response);
          }
        },
        onError: (error) {
          if (error is SocketException && error.osError?.errorCode == 51) {
            GUtils.showToast(
                message: GTexts.plzChkNetwork,
                toastWithDuration: Toast.LENGTH_LONG);
          }
          if (showDialog) Global.navigatorKey.currentState!.pop();
          if (onError != null) onError();
        },
      );

  //Utility function for firing Multi-part request
  static Future<StreamedResponse> sendMultipart(
      {bool showDialog = true,
      required MultipartRequest request,
      Function(Response)? onResponse,
      Function? onError}) {
    if (showDialog) {
      GUtils.showProgressDialog();
    }
    var streamedRespFuture = request.send();
    _attachHandlerToBaseResponseFuture(
        baseRespFuture: streamedRespFuture,
        showDialog: showDialog,
        onResponse: onResponse,
        onError: onError);
    return streamedRespFuture;
  }
}
